/*
 * main.cpp
 *
 *  Created on: Nov 23, 2014
 *      Author: kibaekkim
 */

#include "OsiClpSolverInterface.hpp"
#include "CbcModel.hpp"
#include "CbcStrategy.hpp"

using namespace std;

bool isNumericStr( const char *str )
{
   const size_t l = strlen(str);

   for ( size_t i=0 ; i<l ; ++i )
     if (!(isdigit(str[i])||(str[i]=='.')||(str[i]=='-')))
         return false;

   return true;
}

int readMIPStart( CbcModel * model, const char *fileName,
                  vector< pair< string, double > > &colValues,
                  double &/*solObj*/ )
{
#define STR_SIZE 256
   FILE *f = fopen( fileName, "r" );
   if (!f)
      return 1;
   char line[STR_SIZE];

   int nLine = 0;
   char printLine[STR_SIZE];
   while (fgets( line, STR_SIZE, f ))
   {
      ++nLine;
      char col[4][STR_SIZE];
      int nread = sscanf( line, "%s %s %s %s", col[0], col[1], col[2], col[3] );
      if (!nread)
         continue;
      /* line with variable value */
      if (strlen(col[0])&&isdigit(col[0][0])&&(nread>=3))
      {
         if (!isNumericStr(col[0]))
         {
            sprintf( printLine, "Reading: %s, line %d - first column in mipstart file should be numeric, ignoring.", fileName, nLine );
	    model->messageHandler()->message(CBC_GENERAL, model->messages())
	      << printLine << CoinMessageEol;
            continue;
         }
         if (!isNumericStr(col[2]))
         {
            sprintf( printLine, "Reading: %s, line %d - Third column in mipstart file should be numeric, ignoring.", fileName, nLine  );
	    model->messageHandler()->message(CBC_GENERAL, model->messages())
	      << printLine << CoinMessageEol;
            continue;
         }

         //int idx = atoi( col[0] );
         char *name = col[1];
         double value = atof( col[2] );
         //double obj = 0.0;
//         if (nread >= 4)
//            obj = atof( col[3] );

         colValues.push_back( pair<string, double>(string(name),value) );
      }
   }

   if (colValues.size()) {
      sprintf( printLine,"mipstart values read for %d variables.", (int)colValues.size());
      model->messageHandler()->message(CBC_GENERAL, model->messages())
	<< printLine << CoinMessageEol;
      if (colValues.size()<model->getNumCols()) {
	int numberColumns = model->getNumCols();
	OsiSolverInterface * solver = model->solver();
	vector< pair< string, double > > fullValues;
	/* for fast search of column names */
	map< string, int > colIdx;
	for (int i=0;i<numberColumns;i++) {
	  fullValues.push_back( pair<string, double>(solver->getColName(i),0.0) );
	  colIdx[solver->getColName(i)] = i;
	}
	for ( int i=0 ; (i<(int)colValues.size()) ; ++i )
	  {
	    map< string, int >::const_iterator mIt = colIdx.find( colValues[i].first );
	    if ( mIt != colIdx.end() ) {
	      const int idx = mIt->second;
	      double v = colValues[i].second;
	      fullValues[idx].second=v;
	    }
	  }
	colValues=fullValues;
      }
   } else
   {
      sprintf( printLine, "No mipstart solution read from %s", fileName );
      model->messageHandler()->message(CBC_GENERAL, model->messages())
	<< printLine << CoinMessageEol;
      return 1;
   }

   fclose(f);
   return 0;
}

int main()
{
	OsiClpSolverInterface si;
	si.readMps("mip.mps.gz");

	si.setLogLevel(0);

	CbcModel model(si);
	CbcStrategyDefault strategy(0);         // assertion
	//CbcStrategyDefault strategy(0, 5, 5); // infinite loop
	//CbcStrategyDefault strategy(1);       // success
	model.setStrategy(strategy);

	std::vector< std::pair< std::string, double > > colVals;
	double solObj;
	readMIPStart(&model, "mipstart.txt", colVals, solObj);

	double * solution = new double [model.getNumCols()];
	for (int j = 0; j < model.getNumCols(); ++j)
		solution[j] = colVals[j].second;

	model.setHotstartSolution(solution);

	delete [] solution;

	model.branchAndBound();

	return 0;
}


